@@ -4,33 +4,19 @@ use strict;
 use warnings;
 use SOAP::Lite;
 
-our $VERSION = '0.301';
+our $VERSION = '0.302';
 $VERSION = eval $VERSION;
 
 use base qw( SOAP::Deserializer );
 use strict 'refs';
 
-#**************************************************************************
-# new()
-#   -- constructor
-#**************************************************************************
-sub new {
-    my $class    = shift;
-    my $self     = $class->SUPER::new(@_);
-    my (%params) = @_;
-    return $self;
-}
+our $XSD_NSPREFIX     = "xsd";
+our $XSI_NSPREFIX     = "xsi";
+our $SOAPENV_NSPREFIX = "SOAP-ENV";
+our $SOAPENC_NSPREFIX = "SOAP-ENC";
+our $NSPREFIX         = "wsisup";
 
 BEGIN {
-    use vars qw($XSD_NSPREFIX $XSI_NSPREFIX $SOAPENV_NSPREFIX
-      $SOAPENC_NSPREFIX $NSPREFIX);
-
-    $XSD_NSPREFIX     = "xsd";
-    $XSI_NSPREFIX     = "xsi";
-    $SOAPENV_NSPREFIX = "SOAP-ENV";
-    $SOAPENC_NSPREFIX = "SOAP-ENC";
-    $NSPREFIX         = "wsisup";
-
     no strict 'refs';
     for my $class (qw(LoginResult)) {
         my $method_name = "as_" . $class;
